@@ -297,6 +297,42 @@ impl GLContext {
             .unwrap();
     }
 
+    pub fn tex_sub_image_2d_with_u16_data(
+        &self,
+        target: u32,
+        level: u32,
+        x_offset: u32,
+        y_offset: u32,
+        width: u32,
+        height: u32,
+        format: u32,
+        data_type: DataType,
+        pixels: &[u16],
+    ) {
+        use wasm_bindgen::JsCast;
+        let memory_buffer = wasm_bindgen::memory()
+            .dyn_into::<js_sys::WebAssembly::Memory>()
+            .unwrap()
+            .buffer();
+        let data_location = pixels.as_ptr() as u32 / 2;
+        let array = js_sys::Uint16Array::new(&memory_buffer)
+            .subarray(data_location, data_location + pixels.len() as u32);
+
+        self.inner
+            .tex_sub_image_2d_with_i32_and_i32_and_u32_and_type_and_opt_array_buffer_view(
+                target,
+                level as i32,
+                x_offset as i32,
+                y_offset as i32,
+                width as i32,
+                height as i32,
+                format,
+                data_type.to_const(),
+                Some(&array),
+            )
+            .unwrap();
+    }
+
     pub fn tex_sub_image_2d_with_u32_data(
         &self,
         target: u32,
@@ -464,6 +500,37 @@ impl GLContext {
             .unwrap()
     }
 
+    pub fn read_pixels_with_u16_data(
+        &self,
+        x: u32,
+        y: u32,
+        width: u32,
+        height: u32,
+        format: u32,
+        data_type: DataType,
+        dst_data: &mut [u16],
+    ) {
+        use wasm_bindgen::JsCast;
+        let memory_buffer = wasm_bindgen::memory()
+            .dyn_into::<js_sys::WebAssembly::Memory>()
+            .unwrap()
+            .buffer();
+        let data_location = dst_data.as_ptr() as u32 / 2;
+        let array = js_sys::Uint16Array::new(&memory_buffer)
+            .subarray(data_location, data_location + dst_data.len() as u32);
+        self.inner
+            .read_pixels_with_opt_array_buffer_view(
+                x as i32,
+                y as i32,
+                width as i32,
+                height as i32,
+                format,
+                data_type.to_const(),
+                Some(&array),
+            )
+            .unwrap();
+    }
+
     pub fn read_pixels_with_f32_data(
         &self,
         x: u32,
@@ -738,17 +805,3 @@ impl ShaderType {
         }
     }
 }
-
-impl DataType {
-    fn to_const(&self) -> u32 {
-        match self {
-            DataType::Float => consts::FLOAT,
-            DataType::Byte => consts::BYTE,
-            DataType::UnsignedByte => consts::UNSIGNED_BYTE,
-            DataType::Short => consts::SHORT,
-            DataType::UnsignedShort => consts::UNSIGNED_SHORT,
-            DataType::Int => consts::INT,
-            DataType::UnsignedInt => consts::UNSIGNED_INT,
-        }
-    }
-}
