@@ -3,6 +3,8 @@
 //! Basic math functionality. Mostly just an re-export of [cgmath](https://crates.io/crates/cgmath).
 //!
 
+pub use half::f16;
+
 pub(crate) use cgmath::ortho;
 pub(crate) use cgmath::perspective;
 pub use cgmath::prelude::*;
@@ -105,10 +107,10 @@ impl Mat4Ext for Mat4 {
     }
 }
 
-pub fn degrees(v: f32) -> Degrees {
+pub const fn degrees(v: f32) -> Degrees {
     Deg(v)
 }
-pub fn radians(v: f32) -> Radians {
+pub const fn radians(v: f32) -> Radians {
     Rad(v)
 }
 
