@@ -70,11 +70,11 @@ impl DepthTargetTexture2D {
     /// Before writing, the texture is cleared based on the given clear state.
     ///
     pub fn write<F: FnOnce() -> ThreeDResult<()>>(
-        &self,
+        &mut self,
         clear_state: Option<f32>,
         render: F,
     ) -> ThreeDResult<()> {
-        RenderTarget::<f32>::new_depth(&self.context, &self)?.write(
+        RenderTarget::new_depth(&self.context.clone(), self)?.write(
             ClearState {
                 depth: clear_state,
                 ..ClearState::none()
@@ -89,12 +89,14 @@ impl DepthTargetTexture2D {
     /// # Errors
     /// Will return an error if the destination is a color texture.
     ///
+    #[deprecated = "Use RenderTarget::copy_from or Screen::copy_from instead"]
     pub fn copy_to<T: TextureDataType>(
-        &self,
+        &mut self,
         destination: CopyDestination<T>,
         viewport: Viewport,
     ) -> ThreeDResult<()> {
-        RenderTarget::new_depth(&self.context, &self)?.copy_to(
+        #[allow(deprecated)]
+        RenderTarget::new_depth_internal(&self.context.clone(), self)?.copy_to(
             destination,
             viewport,
             WriteMask::DEPTH,
