@@ -71,15 +71,13 @@ fn parse_tree<'a>(
                     positions.push(value[2]);
                 }
 
-                let normals = reader.read_normals().map(|values| {
-                    let mut nors = Vec::new();
-                    for value in values {
-                        nors.push(value[0]);
-                        nors.push(value[1]);
-                        nors.push(value[2]);
-                    }
-                    nors
-                });
+                let normals = reader
+                    .read_normals()
+                    .map(|values| values.flatten().collect::<Vec<_>>());
+
+                let tangents = reader
+                    .read_tangents()
+                    .map(|values| values.flatten().collect::<Vec<_>>());
 
                 let indices = reader.read_indices().map(|values| match values {
                     ::gltf::mesh::util::ReadIndices::U8(iter) => {
@@ -152,6 +150,11 @@ fn parse_tree<'a>(
                         } else {
                             (None, 1.0)
                         };
+                    let emissive_texture = if let Some(info) = material.emissive_texture() {
+                        Some(parse_texture(loaded, path, buffers, info.texture())?)
+                    } else {
+                        None
+                    };
                     cpu_materials.push(CPUMaterial {
                         name: material_name.clone(),
                         albedo: Color::from_rgba_slice(&color),
@@ -164,6 +167,9 @@ fn parse_tree<'a>(
                         occlusion_texture,
                         occlusion_strength,
                         occlusion_metallic_roughness_texture: None,
+                        emissive: Color::from_rgb_slice(&material.emissive_factor()),
+                        emissive_texture,
+                        alpha_cutout: None,
                     });
                 }
 
@@ -190,6 +196,7 @@ fn parse_tree<'a>(
                     name: name.clone(),
                     positions,
                     normals,
+                    tangents,
                     indices,
                     colors,
                     uvs,
