@@ -20,6 +20,10 @@ mod ambient_light;
 #[doc(inline)]
 pub use ambient_light::*;
 
+mod environment;
+#[doc(inline)]
+pub use environment::*;
+
 use crate::core::*;
 
 #[derive(Debug, Copy, Clone, PartialEq)]
@@ -41,6 +45,22 @@ pub enum NormalDistributionFunction {
     TrowbridgeReitzGGX,
 }
 
+impl LightingModel {
+    pub(crate) fn shader(&self) -> &str {
+        match self {
+            LightingModel::Phong => "#define PHONG",
+            LightingModel::Blinn => "#define BLINN",
+            LightingModel::Cook(normal, _) => match normal {
+                NormalDistributionFunction::Blinn => "#define COOK\n#define COOK_BLINN\n",
+                NormalDistributionFunction::Beckmann => "#define COOK\n#define COOK_BECKMANN\n",
+                NormalDistributionFunction::TrowbridgeReitzGGX => {
+                    "#define COOK\n#define COOK_GGX\n"
+                }
+            },
+        }
+    }
+}
+
 pub struct Lights {
     pub ambient: Option<AmbientLight>,
     pub directional: Vec<DirectionalLight>,
@@ -55,11 +75,16 @@ impl Lights {
     }
 
     pub fn use_uniforms(&self, program: &Program, camera: &Camera) -> ThreeDResult<()> {
+        program.use_uniform_vec3("eyePosition", camera.position())?;
         for (i, light) in LightsIterator::new(self).enumerate() {
-            light.use_uniforms(program, camera, i as u32)?;
+            light.use_uniforms(program, i as u32)?;
         }
         Ok(())
     }
+
+    pub fn iter<'a>(&'a self) -> LightsIterator<'a> {
+        LightsIterator::new(self)
+    }
 }
 
 impl Default for Lights {
@@ -74,7 +99,7 @@ impl Default for Lights {
     }
 }
 
-struct LightsIterator<'a> {
+pub struct LightsIterator<'a> {
     lights: &'a Lights,
     index: usize,
 }
@@ -88,13 +113,16 @@ impl<'a> LightsIterator<'a> {
 impl<'a> Iterator for LightsIterator<'a> {
     type Item = &'a dyn Light;
     fn next(&mut self) -> Option<Self::Item> {
+        let mut count = 0;
         let result = self
             .lights
             .ambient
             .as_ref()
             .filter(|_| self.index == 0)
             .map(|l| l as &dyn Light);
-        let mut count = if self.lights.ambient.is_some() { 1 } else { 0 };
+        if self.lights.ambient.is_some() {
+            count += 1;
+        }
 
         let result = result.or_else(|| {
             self.lights
@@ -128,28 +156,21 @@ pub(crate) fn lights_fragment_shader_source(
     lights: &mut dyn Iterator<Item = &dyn Light>,
     lighting_model: LightingModel,
 ) -> String {
-    let mut shader_source = match lighting_model {
-        LightingModel::Phong => "#define PHONG",
-        LightingModel::Blinn => "#define BLINN",
-        LightingModel::Cook(normal, _) => match normal {
-            NormalDistributionFunction::Blinn => "#define COOK\n#define COOK_BLINN\n",
-            NormalDistributionFunction::Beckmann => "#define COOK\n#define COOK_BECKMANN\n",
-            NormalDistributionFunction::TrowbridgeReitzGGX => "#define COOK\n#define COOK_GGX\n",
-        },
-    }
-    .to_string();
+    let mut shader_source = lighting_model.shader().to_string();
     shader_source.push_str(include_str!("../core/shared.frag"));
-    shader_source.push_str(include_str!("./material/shaders/light_shared.frag"));
+    shader_source.push_str(include_str!("./light/shaders/light_shared.frag"));
     let mut dir_fun = String::new();
     for (i, light) in lights.enumerate() {
         shader_source.push_str(&light.shader_source(i as u32));
-        dir_fun.push_str(&format!("color += calculate_lighting{}(surface_color, position, normal, metallic, roughness, occlusion);\n", i))
+        dir_fun.push_str(&format!("color += calculate_lighting{}(surface_color, position, normal, view_direction, metallic, roughness, occlusion);\n", i))
     }
     shader_source.push_str(&format!(
         "
+            uniform vec3 eyePosition;
             vec3 calculate_lighting(vec3 surface_color, vec3 position, vec3 normal, float metallic, float roughness, float occlusion)
             {{
                 vec3 color = vec3(0.0, 0.0, 0.0);
+                vec3 view_direction = normalize(eyePosition - position);
                 {}
                 return color;
             }}
@@ -161,15 +182,15 @@ pub(crate) fn lights_fragment_shader_source(
 
 pub trait Light {
     fn shader_source(&self, i: u32) -> String;
-    fn use_uniforms(&self, program: &Program, camera: &Camera, i: u32) -> ThreeDResult<()>;
+    fn use_uniforms(&self, program: &Program, i: u32) -> ThreeDResult<()>;
 }
 
-impl<T: Light> Light for &T {
+impl<T: Light + ?Sized> Light for &T {
     fn shader_source(&self, i: u32) -> String {
         (*self).shader_source(i)
     }
-    fn use_uniforms(&self, program: &Program, camera: &Camera, i: u32) -> ThreeDResult<()> {
-        (*self).use_uniforms(program, camera, i)
+    fn use_uniforms(&self, program: &Program, i: u32) -> ThreeDResult<()> {
+        (*self).use_uniforms(program, i)
     }
 }
 
