@@ -19,7 +19,7 @@ impl SpotLight {
         color: Color,
         position: &Vec3,
         direction: &Vec3,
-        cutoff: f32,
+        cutoff: impl Into<Radians>,
         attenuation_constant: f32,
         attenuation_linear: f32,
         attenuation_exponential: f32,
@@ -83,12 +83,12 @@ impl SpotLight {
         vec3(p[0], p[1], p[2])
     }
 
-    pub fn set_cutoff(&mut self, cutoff: f32) {
-        self.light_buffer.update(7, &[cutoff]).unwrap();
+    pub fn set_cutoff(&mut self, cutoff: impl Into<Radians>) {
+        self.light_buffer.update(7, &[cutoff.into().0]).unwrap();
     }
 
-    pub fn cutoff(&self) -> f32 {
-        self.light_buffer.get(7).unwrap()[0]
+    pub fn cutoff(&self) -> Radians {
+        radians(self.light_buffer.get(7).unwrap()[0])
     }
 
     pub fn set_direction(&mut self, direction: &Vec3) {
@@ -107,32 +107,41 @@ impl SpotLight {
         self.light_buffer.update(9, &[0.0]).unwrap();
     }
 
-    pub fn generate_shadow_map<G: Geometry>(
+    pub fn generate_shadow_map(
         &mut self,
-        frustrum_depth: f32,
         texture_size: u32,
-        geometries: &[G],
+        geometries: &[impl Geometry],
     ) -> ThreeDResult<()> {
         let position = self.position();
         let direction = self.direction();
         let up = compute_up_direction(direction);
-        let cutoff = self.light_buffer.get(7).unwrap()[0];
 
         let viewport = Viewport::new_at_origo(texture_size, texture_size);
+
+        let mut z_far = 0.0f32;
+        let mut z_near = f32::MAX;
+        for geometry in geometries {
+            let aabb = geometry.aabb();
+            if !aabb.is_empty() {
+                z_far = z_far.max(aabb.distance_max(&position));
+                z_near = z_near.min(aabb.distance(&position));
+            }
+        }
+
         let shadow_camera = Camera::new_perspective(
             &self.context,
             viewport,
             position,
             position + direction,
             up,
-            degrees(cutoff),
-            0.1,
-            frustrum_depth,
+            self.cutoff(),
+            z_near.max(0.01),
+            z_far,
         )?;
         self.light_buffer
             .update(10, &shadow_matrix(&shadow_camera).to_slice())?;
 
-        let shadow_texture = DepthTargetTexture2D::new(
+        let mut shadow_texture = DepthTargetTexture2D::new(
             &self.context,
             texture_size,
             texture_size,
@@ -152,7 +161,11 @@ impl SpotLight {
                 .iter()
                 .filter(|g| shadow_camera.in_frustum(&g.aabb()))
             {
-                geometry.render_forward(&depth_material, &shadow_camera, &Lights::default())?;
+                geometry.render_with_material(
+                    &depth_material,
+                    &shadow_camera,
+                    &Lights::default(),
+                )?;
             }
             Ok(())
         })?;
@@ -185,18 +198,22 @@ impl Light for SpotLight {
                 float shadowEnabled{};
                 mat4 shadowMVP{};
             }};
-            vec3 calculate_lighting{}(vec3 surface_color, vec3 position, vec3 normal, float metallic, float roughness, float occlusion)
+            vec3 calculate_lighting{}(vec3 surface_color, vec3 position, vec3 normal, vec3 view_direction, float metallic, float roughness, float occlusion)
             {{
                 if(base{}.intensity > 0.001) {{
-                    vec3 light_color = base{}.intensity * base{}.color;
-                    vec3 light_direction = normalize(position - position{});
-                    float angle = acos(dot(light_direction, normalize(direction{})));
-                    float cutoff = 3.14 * cutoff{} / 180.0;
+                    vec3 light_direction = position{} - position;
+                    float distance = length(light_direction);
+                    light_direction = light_direction / distance;
+
+                    float angle = acos(dot(-light_direction, normalize(direction{})));
+                    float cutoff = cutoff{};
                 
                     vec3 result = vec3(0.0);
                     if (angle < cutoff) {{
-                        result = calculate_attenuated_light(light_color, attenuation{}, position{}, surface_color, position, normal, 
-                            metallic, roughness, occlusion) * (1.0 - smoothstep(0.75 * cutoff, cutoff, angle));
+    
+                        vec3 light_color = attenuate(base{}.intensity * base{}.color, attenuation{}, distance);
+                        result = calculate_light(light_color, light_direction, surface_color, view_direction, normal, 
+                            metallic, roughness) * (1.0 - smoothstep(0.75 * cutoff, cutoff, angle));
                         if(shadowEnabled{} > 0.5) {{
                             result *= calculate_shadow(shadowMap{}, shadowMVP{}, position);
                         }}
@@ -208,16 +225,15 @@ impl Light for SpotLight {
                 }}
             }}
         
-        ", i, i, i, i, i, i, i, i, i, i, i, i, i, i, i, i, i, i, i, i, i)
+        ", i, i, i, i, i, i, i, i, i, i, i, i, i, i, i, i, i, i, i, i)
     }
-    fn use_uniforms(&self, program: &Program, camera: &Camera, i: u32) -> ThreeDResult<()> {
+    fn use_uniforms(&self, program: &Program, i: u32) -> ThreeDResult<()> {
         if let Some(tex) = self.shadow_map() {
             program.use_texture(&format!("shadowMap{}", i), tex)?;
         } else {
             self.context
                 .use_texture_dummy(&program, &format!("shadowMap{}", i))?;
         }
-        program.use_uniform_vec3("eyePosition", camera.position())?;
         program.use_uniform_block(&format!("LightUniform{}", i), self.buffer());
         Ok(())
     }
